package lcsb.mapviewer.converter.model.sbgnml;

import java.awt.Desktop;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.nio.file.Files;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.LogEvent;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;
import org.sbgn.GlyphClazz;
import org.sbgn.bindings.Bbox;
import org.sbgn.bindings.Glyph;

import lcsb.mapviewer.common.MinervaLoggerAppender;
import lcsb.mapviewer.common.tests.TestUtils;
import lcsb.mapviewer.common.tests.UnitTestFailedWatcher;
import lcsb.mapviewer.converter.Converter;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.converter.InvalidInputDataExecption;
import lcsb.mapviewer.converter.graphics.AbstractImageGenerator;
import lcsb.mapviewer.converter.graphics.NormalImageGenerator;
import lcsb.mapviewer.converter.graphics.PngImageGenerator;
import lcsb.mapviewer.model.graphics.HorizontalAlign;
import lcsb.mapviewer.model.graphics.VerticalAlign;
import lcsb.mapviewer.model.map.InconsistentModelException;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.GenericProtein;

public class SbgnmlTestFunctions extends TestUtils {

  protected static Logger logger = LogManager.getLogger();

  @Rule
  public UnitTestFailedWatcher unitTestFailedWatcher = new UnitTestFailedWatcher();

  private MinervaLoggerAppender appender;

  private static int counter = 0;

  @Before
  public final void _setUp() throws Exception {
    MinervaLoggerAppender.unregisterLogEventStorage(appender);
    appender = MinervaLoggerAppender.createAppender();
  }

  @After
  public final void _tearDown() throws Exception {
    MinervaLoggerAppender.unregisterLogEventStorage(appender);
  }

  protected List<LogEvent> getWarnings() {
    return appender.getWarnings();
  }

  protected void showImage(final Model model) throws Exception {
    String dir = Files.createTempDirectory("sbml-temp-images-dir").toFile().getAbsolutePath();
    AbstractImageGenerator.Params params = new AbstractImageGenerator.Params().height(model.getHeight())
        .width(model.getWidth()).nested(true).scale(1).level(20).x(0).y(0).model(model);
    NormalImageGenerator nig = new PngImageGenerator(params);
    String pathWithouExtension = dir + "/" + model.getName();
    String pngFilePath = pathWithouExtension.concat(".png");
    nig.saveToFile(pngFilePath);
    Desktop.getDesktop().open(new File(pngFilePath));
  }

  protected Glyph createGlyph(final GlyphClazz clazz) {
    Glyph glyph = new Glyph();
    Bbox bbox = new Bbox();
    bbox.setX(10);
    bbox.setY(20);
    bbox.setW(30);
    bbox.setH(40);
    glyph.setBbox(bbox);
    glyph.setClazz(clazz.getClazz());
    return glyph;
  }

  protected GenericProtein createProtein() {
    GenericProtein protein = new GenericProtein("id" + counter++);
    protein.setWidth(50);
    protein.setHeight(30);
    protein.setX(200);
    protein.setX(300);
    protein.setZ(1);
    protein.setNameWidth(protein.getWidth());
    protein.setNameHeight(protein.getHeight());
    protein.setNameX(protein.getX());
    protein.setNameY(protein.getY());
    protein.setNameVerticalAlign(VerticalAlign.MIDDLE);
    protein.setNameHorizontalAlign(HorizontalAlign.CENTER);
    return protein;
  }

  protected Model serializeOverSbgn(final Model model) throws InconsistentModelException, ConverterException, InvalidInputDataExecption {
    Converter converter = new SbgnmlXmlConverter();
    String xml = converter.model2String(model);
    Model model2 = converter.createModel(new ConverterParams().inputStream(new ByteArrayInputStream(xml.getBytes())).sizeAutoAdjust(false));
    model2.setIdModel(model.getIdModel());
    return model2;
  }

}
