package lcsb.mapviewer.wikipathway;

import java.awt.Color;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.core.LogEvent;
import org.junit.After;
import org.junit.Before;
import org.junit.Rule;

import lcsb.mapviewer.common.MinervaLoggerAppender;
import lcsb.mapviewer.common.tests.TestUtils;
import lcsb.mapviewer.common.tests.UnitTestFailedWatcher;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.converter.ConverterParams;
import lcsb.mapviewer.converter.InvalidInputDataExecption;
import lcsb.mapviewer.converter.ZIndexPopulator;
import lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser;
import lcsb.mapviewer.model.graphics.HorizontalAlign;
import lcsb.mapviewer.model.graphics.PolylineData;
import lcsb.mapviewer.model.graphics.VerticalAlign;
import lcsb.mapviewer.model.map.Drawable;
import lcsb.mapviewer.model.map.InconsistentModelException;
import lcsb.mapviewer.model.map.compartment.Compartment;
import lcsb.mapviewer.model.map.layout.graphics.Layer;
import lcsb.mapviewer.model.map.layout.graphics.LayerRect;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Species;

public abstract class WikipathwaysTestFunctions extends TestUtils {

  protected static Logger logger = LogManager.getLogger();

  @Rule
  public UnitTestFailedWatcher unitTestFailedWatcher = new UnitTestFailedWatcher();

  private MinervaLoggerAppender appender;

  @Before
  public final void _setUp() throws Exception {
    MinervaLoggerAppender.unregisterLogEventStorage(appender);
    appender = MinervaLoggerAppender.createAppender();
  }

  @After
  public final void _tearDown() throws Exception {
    MinervaLoggerAppender.unregisterLogEventStorage(appender);
  }

  protected List<LogEvent> getWarnings() {
    return appender.getWarnings();
  }

  protected Model serializeModelOverCellDesignerParser(final Model model1)
      throws InconsistentModelException, InvalidInputDataExecption {
    for (final Drawable bioEntity : model1.getDrawables()) {
      if (!(bioEntity instanceof PolylineData)) {
        bioEntity.setZ(null);
      }
    }
    for (lcsb.mapviewer.model.map.species.Element bioEntity : model1.getElements()) {
      bioEntity.setFontColor(Color.BLACK);
    }

    // CellDesigner doesn't allow to store fill color for rectangles
    for (final Layer layer : model1.getLayers()) {
      for (final LayerRect rect : layer.getRectangles()) {
        rect.setFillColor(Color.LIGHT_GRAY);
      }
    }
    // CellDesigner doesn't allow to store compartment font size
    for (final Compartment bioEntity : model1.getCompartments()) {
      bioEntity.setFontSize(Compartment.DEFAULT_COMPARTMENT_THICKNESS);
    }
    new ZIndexPopulator().populateZIndex(model1);
    CellDesignerXmlParser parser = new CellDesignerXmlParser();
    String xml = parser.model2String(model1);
    InputStream is = new ByteArrayInputStream(xml.getBytes(StandardCharsets.UTF_8));

    for (lcsb.mapviewer.model.map.species.Element bioEntity : model1.getElements()) {
      // CellDesigner doesn't allow to store border color
      if (bioEntity instanceof Species) {
        bioEntity.setBorderColor(Color.BLACK);
      }
      // CellDesigner doesn't allow to store name point position
      bioEntity.setNameX(0);
      bioEntity.setNameY(0);
      bioEntity.setNameWidth(0);
      bioEntity.setNameHeight(0);
      bioEntity.setNameHorizontalAlign(null);
      bioEntity.setNameVerticalAlign(null);
    }

    Model model2 = parser.createModel(new ConverterParams().inputStream(is).sizeAutoAdjust(false));
    for (lcsb.mapviewer.model.map.species.Element bioEntity : model2.getElements()) {
      // CellDesigner doesn't allow to store name point position
      bioEntity.setNameX(0);
      bioEntity.setNameY(0);
      bioEntity.setNameWidth(0);
      bioEntity.setNameHeight(0);
      bioEntity.setNameHorizontalAlign(null);
      bioEntity.setNameVerticalAlign(null);
    }
    return model2;
  }

  private int z = 10;

  protected void assignCoordinates(final double x, final double y, final double width, final double height,
      final lcsb.mapviewer.model.map.species.Element element) {
    element.setX(x);
    element.setY(y);
    element.setZ(z++);
    element.setWidth(width);
    element.setHeight(height);
    element.setNameX(x);
    element.setNameY(y);
    element.setNameWidth(width);
    element.setNameHeight(height);
    element.setNameVerticalAlign(VerticalAlign.MIDDLE);
    element.setNameHorizontalAlign(HorizontalAlign.CENTER);
  }

  protected Model serializeOverGpml(final Model model)
      throws InconsistentModelException, ConverterException, InvalidInputDataExecption {
    lcsb.mapviewer.wikipathway.GpmlParser parser = new lcsb.mapviewer.wikipathway.GpmlParser();

    String xml = parser.model2String(model);
    Model model2 = parser.createModel(new ConverterParams().inputStream(new ByteArrayInputStream(xml.getBytes())));
    return model2;
  }

}
